@@ -1,5 +1,5 @@
 /* Test of POSIX compatible fflush() function.
-   Copyright (C) 2008-2013 Free Software Foundation, Inc.
+   Copyright (C) 2008-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -28,7 +28,7 @@ main (int argc, char **argv)
 
   /* Avoid the well-known bugs of fflush() on streams in O_TEXT mode
      on native Windows platforms.  */
-  SET_BINARY (0);
+  set_binary_mode (0, O_BINARY);
 
   if (argc > 1)
     switch (argv[1][0])
@@ -36,7 +36,7 @@ main (int argc, char **argv)
       case '1':
         /* Check fflush after a backup ungetc() call.  This is case 1a in
            terms of
-           <http://lists.gnu.org/archive/html/bug-gnulib/2008-03/msg00131.html>,
+           <https://lists.gnu.org/r/bug-gnulib/2008-03/msg00131.html>,
            according to the Austin Group's resolution on 2009-01-08.  */
 
         c = fgetc (stdin);
@@ -65,20 +65,14 @@ main (int argc, char **argv)
       case '2':
         /* Check fflush after a non-backup ungetc() call.  This is case 2a in
            terms of
-           <http://lists.gnu.org/archive/html/bug-gnulib/2008-03/msg00131.html>,
+           <https://lists.gnu.org/r/bug-gnulib/2008-03/msg00131.html>,
            according to the Austin Group's resolution on 2009-01-08.  */
         /* Check that fflush after a non-backup ungetc() call discards the
            ungetc buffer.  This is mandated by POSIX
-           <http://www.opengroup.org/susv3/functions/ungetc.html>:
-             "The value of the file-position indicator for the stream after
-              reading or discarding all pushed-back bytes shall be the same
-              as it was before the bytes were pushed back."
-           <http://www.opengroup.org/austin/aardvark/latest/xshbug3.txt>
-             "[After fflush(),] the file offset of the underlying open file
-              description shall be set to the file position of the stream, and
-              any characters pushed back onto the stream by ungetc() or
-              ungetwc() that have not subsequently been read from the stream
-              shall be discarded."  */
+           <https://pubs.opengroup.org/onlinepubs/9699919799/functions/fflush.html>:
+             "...any characters pushed back onto the stream by ungetc()
+              or ungetwc() that have not subsequently been read from the
+              stream shall be discarded..."  */
 
         c = fgetc (stdin);
         ASSERT (c == '#');
