@@ -1,6 +1,6 @@
 /* Host name canonicalization
 
-   Copyright (C) 2005-2018 Free Software Foundation, Inc.
+   Copyright (C) 2005-2020 Free Software Foundation, Inc.
 
    Written by Derek Price <derek@ximbiot.com>.
 
@@ -52,7 +52,8 @@ canon_host (const char *host)
      such value for any given hostname, though the actual IP address
      referenced by this value and the device using that IP address may each
      actually have any number of such "canonical" hostnames.  See the POSIX
-     getaddrinfo spec <http://www.opengroup.org/susv3xsh/getaddrinfo.html">,
+     getaddrinfo spec
+     <https://pubs.opengroup.org/onlinepubs/9699919799/functions/getaddrinfo.html>,
      RFC 1034 <https://www.ietf.org/rfc/rfc1034.txt>, & RFC 2181
      <https://www.ietf.org/rfc/rfc2181.txt> for more on what this confusing
      term really refers to. */
