@@ -1,6 +1,6 @@
 /* xnanosleep.c -- a more convenient interface to nanosleep
 
-   Copyright (C) 2002-2007, 2009-2018 Free Software Foundation, Inc.
+   Copyright (C) 2002-2007, 2009-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,10 +22,12 @@
 
 #include "xnanosleep.h"
 
+#include <intprops.h>
 #include <timespec.h>
 
 #include <errno.h>
 #include <time.h>
+#include <unistd.h>
 
 /* Sleep until the time (call it WAKE_UP_TIME) specified as
    SECONDS seconds after the time this function is called.
@@ -37,6 +39,17 @@
 int
 xnanosleep (double seconds)
 {
+#if HAVE_PAUSE
+  if (1.0 + TYPE_MAXIMUM (time_t) <= seconds)
+    {
+      do
+        pause ();
+      while (errno == EINTR);
+
+      /* pause failed (!); fall back on repeated nanosleep calls.  */
+    }
+#endif
+
   struct timespec ts_sleep = dtotimespec (seconds);
 
   for (;;)
@@ -46,9 +59,14 @@ xnanosleep (double seconds)
          set errno to EINTR.  nanosleep from linux-2.6.10, as well as
          implementations by (all?) other vendors, doesn't return -1
          in that case;  either it continues sleeping (if time remains)
-         or it returns zero (if the wake-up time has passed).  */
+         or it returns zero (if the wake-up time has passed).
+
+         Gnulib's replacement nanosleep sometimes does not update
+         TS_SLEEP, and it is possible some kernels have a similar bug.
+         However, this merely causes xnanosleep to sleep longer than
+         necessary, which is not a correctness bug.  */
       errno = 0;
-      if (nanosleep (&ts_sleep, NULL) == 0)
+      if (nanosleep (&ts_sleep, &ts_sleep) == 0)
         break;
       if (errno != EINTR && errno != 0)
         return -1;
