@@ -1,6 +1,6 @@
 /* xtime -- extended-resolution integer timestamps
 
-   Copyright (C) 2005-2006, 2009-2017 Free Software Foundation, Inc.
+   Copyright (C) 2005-2006, 2009-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,37 +29,27 @@ _GL_INLINE_HEADER_BEGIN
 #endif
 
 /* xtime_t is a signed type used for timestamps.  It is an integer
-   type that is a count of nanoseconds -- except for obsolescent hosts
-   without sufficiently-wide integers, where it is a count of
-   seconds.  */
-#if HAVE_LONG_LONG_INT
+   type that is a count of nanoseconds.  */
 typedef long long int xtime_t;
-# define XTIME_PRECISION 1000000000
-#else
-# include <limits.h>
-typedef long int xtime_t;
-# if LONG_MAX >> 31 >> 31 == 0
-#  define XTIME_PRECISION 1
-# else
-#  define XTIME_PRECISION 1000000000
-# endif
-#endif
+#define XTIME_PRECISION 1000000000
 
 #ifdef  __cplusplus
 extern "C" {
 #endif
 
 /* Return an extended time value that contains S seconds and NS
-   nanoseconds, without any overflow checking.  */
+   nanoseconds.  S and NS should be nonnegative; otherwise, integer
+   overflow can occur even if the result is in range.  */
 XTIME_INLINE xtime_t
 xtime_make (xtime_t s, long int ns)
 {
-  if (XTIME_PRECISION == 1)
-    return s;
-  else
-    return XTIME_PRECISION * s + ns;
+  return XTIME_PRECISION * s + ns;
 }
 
+/* The following functions split an extended time value:
+   T = XTIME_PRECISION * xtime_sec (T) + xtime_nsec (T)
+   with  0 <= xtime_nsec (T) < XTIME_PRECISION.  */
+
 /* Return the number of seconds in T, which must be nonnegative.  */
 XTIME_INLINE xtime_t
 xtime_nonnegative_sec (xtime_t t)
@@ -71,11 +61,7 @@ xtime_nonnegative_sec (xtime_t t)
 XTIME_INLINE xtime_t
 xtime_sec (xtime_t t)
 {
-  return (XTIME_PRECISION == 1
-          ? t
-          : t < 0
-          ? (t + XTIME_PRECISION - 1) / XTIME_PRECISION - 1
-          : xtime_nonnegative_sec (t));
+  return (t + (t < 0)) / XTIME_PRECISION - (t < 0);
 }
 
 /* Return the number of nanoseconds in T, which must be nonnegative.  */
