@@ -13,6 +13,10 @@
    https://blake2.net.
 */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,7 +30,59 @@
 
 #include "blake2.h"
 
+#if 0
 /* This will help compatibility with coreutils */
+int blake2s_stream( FILE *stream, void *resstream, size_t outbytes )
+{
+  int ret = -1;
+  size_t sum, n;
+  blake2s_state S[1];
+  static const size_t buffer_length = 32768;
+  uint8_t *buffer = ( uint8_t * )malloc( buffer_length );
+
+  if( !buffer ) return -1;
+
+  blake2s_init( S, outbytes );
+
+  while( 1 )
+  {
+    sum = 0;
+
+    while( 1 )
+    {
+      n = fread( buffer + sum, 1, buffer_length - sum, stream );
+      sum += n;
+
+      if( buffer_length == sum )
+        break;
+
+      if( 0 == n )
+      {
+        if( ferror( stream ) )
+          goto cleanup_buffer;
+
+        goto final_process;
+      }
+
+      if( feof( stream ) )
+        goto final_process;
+    }
+
+    blake2s_update( S, buffer, buffer_length );
+  }
+
+final_process:;
+
+  if( sum > 0 ) blake2s_update( S, buffer, sum );
+
+  blake2s_final( S, resstream, outbytes );
+  ret = 0;
+cleanup_buffer:
+  free( buffer );
+  return ret;
+}
+#endif
+
 int blake2b_stream( FILE *stream, void *resstream, size_t outbytes )
 {
   int ret = -1;
@@ -78,56 +134,6 @@ cleanup_buffer:
 }
 
 #if 0
-int blake2s_stream( FILE *stream, void *resstream, size_t outbytes )
-{
-  int ret = -1;
-  size_t sum, n;
-  blake2s_state S[1];
-  static const size_t buffer_length = 32768;
-  uint8_t *buffer = ( uint8_t * )malloc( buffer_length );
-
-  if( !buffer ) return -1;
-
-  blake2s_init( S, outbytes );
-
-  while( 1 )
-  {
-    sum = 0;
-
-    while( 1 )
-    {
-      n = fread( buffer + sum, 1, buffer_length - sum, stream );
-      sum += n;
-
-      if( buffer_length == sum )
-        break;
-
-      if( 0 == n )
-      {
-        if( ferror( stream ) )
-          goto cleanup_buffer;
-
-        goto final_process;
-      }
-
-      if( feof( stream ) )
-        goto final_process;
-    }
-
-    blake2s_update( S, buffer, buffer_length );
-  }
-
-final_process:;
-
-  if( sum > 0 ) blake2s_update( S, buffer, sum );
-
-  blake2s_final( S, resstream, outbytes );
-  ret = 0;
-cleanup_buffer:
-  free( buffer );
-  return ret;
-}
-
 
 int blake2sp_stream( FILE *stream, void *resstream, size_t outbytes )
 {
@@ -249,6 +255,7 @@ static void usage( char **argv, int errcode )
   exit( errcode );
 }
 
+
 int main( int argc, char **argv )
 {
   blake2fn blake2_stream = blake2b_stream;
