@@ -1,5 +1,5 @@
 /* system-dependent definitions for coreutils
-   Copyright (C) 1989-2019 Free Software Foundation, Inc.
+   Copyright (C) 1989-2020 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -245,15 +245,15 @@ uid_t getuid (void);
    the third argument to x2nrealloc would be 'sizeof *(P)'.
    Ensure that sizeof *(P) is *not* 1.  In that case, it'd be
    better to use X2REALLOC, although not strictly necessary.  */
-#define X2NREALLOC(P, PN) ((void) verify_true (sizeof *(P) != 1), \
-                           x2nrealloc (P, PN, sizeof *(P)))
+#define X2NREALLOC(P, PN) verify_expr (sizeof *(P) != 1, \
+                                       x2nrealloc (P, PN, sizeof *(P)))
 
 /* Using x2realloc (when appropriate) usually makes your code more
    readable than using x2nrealloc, but it also makes it so your
    code will malfunction if sizeof *(P) ever becomes 2 or greater.
    So use this macro instead of using x2realloc directly.  */
-#define X2REALLOC(P, PN) ((void) verify_true (sizeof *(P) == 1), \
-                          x2realloc (P, PN))
+#define X2REALLOC(P, PN) verify_expr (sizeof *(P) == 1, \
+                                      x2realloc (P, PN))
 
 #include "unlocked-io.h"
 #include "same-inode.h"
@@ -285,7 +285,9 @@ readdir_ignoring_dot_and_dotdot (DIR *dirp)
     }
 }
 
-/* Return true if DIR is determined to be an empty directory.  */
+/* Return true if DIR is determined to be an empty directory.
+   Return false with ERRNO==0 if DIR is a non empty directory.
+   Return false if not able to determine if directory empty.  */
 static inline bool
 is_empty_dir (int fd_cwd, char const *dir)
 {
@@ -310,6 +312,7 @@ is_empty_dir (int fd_cwd, char const *dir)
   dp = readdir_ignoring_dot_and_dotdot (dirp);
   saved_errno = errno;
   closedir (dirp);
+  errno = saved_errno;
   if (dp != NULL)
     return false;
   return saved_errno == 0 ? true : false;
@@ -567,11 +570,11 @@ is_nul (void const *buf, size_t length)
 #define DECIMAL_DIGIT_ACCUMULATE(Accum, Digit_val, Type)		\
   (									\
    (void) (&(Accum) == (Type *) NULL),  /* The type matches.  */	\
-   (void) verify_true (! TYPE_SIGNED (Type)), /* The type is unsigned.  */ \
-   (void) verify_true (sizeof (Accum) == sizeof (Type)), /* Added check.  */ \
-   (((Type) -1 / 10 < (Accum)						\
-     || (Type) ((Accum) * 10 + (Digit_val)) < (Accum))			\
-    ? false : (((Accum) = (Accum) * 10 + (Digit_val)), true))		\
+   verify_expr (! TYPE_SIGNED (Type), /* The type is unsigned.  */      \
+                (((Type) -1 / 10 < (Accum)                              \
+                  || (Type) ((Accum) * 10 + (Digit_val)) < (Accum))     \
+                 ? false                                                \
+                 : (((Accum) = (Accum) * 10 + (Digit_val)), true)))     \
   )
 
 static inline void
